<?php

  // --------------------------------------------------------------------------------------

  $on_host = strtoupper( substr( PHP_OS, 0, 3)) === 'WIN' ? false : true;
  $prefix  = '';
  $caller  = 'set_playlist';

  if( $on_host)
  {
    require_once( '../assert_stream_op.php');
    $prefix = '../../playlist_stuff/';
  }


  if( ! defined( 'IN_PLAYLIST'))
  {
    define( 'IN_PLAYLIST', 1);
  }
  require_once( $prefix . 'playlist_settings.php');
  require_once( $prefix . 'playlist_tools.php'   );

  $bh_debug = true;            // Allow for debug info to clutter up the page

  // include( "assert_stream_op.php");


  bh_set_uncached();           // This file works on live data, so it should not be cached

  // --------------------------------------------------------------------------------------

  $debug_info   = "";

  function bh_local_debug( $msg)      // Debug a bit more tidy than with bh_debug();
  {
    global $debug_info;

    $debug_info .= "$msg<br>\n";
  }


  // --------------------------------------------------------------------------------------
  // Auto config stuff

  $this_file         = $caller;
  $caller            = 'undefined now';                  // undefine caller
  $this_url          = $_SERVER[ "SCRIPT_NAME"];         // Get name of this file
  $parts             = explode( '/', $this_url);
  $this_url          = $parts[ count( $parts) - 1];      // bh_local_debug( "this url ' $this_url'");
  $bh_playlists      = bh_load_playlists( $bh_playlists_dir);
  $playlist_pointers = bh_load_playlist_pointers( $bh_playlist_data_fn);

  $default_author   = '';
  $default_playlist = '';

  $filler_author    = '';
  $filler_playlist  = '';

  // --------------------------------------------------------------------------------------
  //
  // Process POST vars, if count seems reasonable and the arguments seem ok
  // save it to file and then generate some contents from that file.

  $count = bh_read_post_param( 'count', 5, -1);

  if( is_numeric( $count) && ( $count >= 0) && ( $count <= $bh_max_playlsit_entries + 1)) // + 1, for default item
  {
    $lines            = '';                   // File contents to be generated
    $n_time           = "0000";               // default GMT
    $n_duration       = 60;                   // default duration in minutes
    $pointers_changed = false;                // Set to true when any playlist counters were cleared

    $lines .= "// visible, playable, day, schedule_type, anchor_date_option, start_time, duration, playlist_author, playlist_name, auto_reset, day_mask\n";

    $default_author   = bh_read_post_param( 'default_author'  , 80, '');
    $default_playlist = bh_read_post_param( 'default_playlist', 80, '');

    $filler_author    = bh_read_post_param( 'filler_author'   , 80, '');
    $filler_playlist  = bh_read_post_param( 'filler_playlist' , 80, '');

    $lines .= "\"default\", \"$default_author\", \"$default_playlist\"\n";
    $lines .= "\"filler\", \"$filler_author\", \"$filler_playlist\"\n";

    for( $i = 0; $i < $count; $i++)
    {
      $is_visible = bh_read_post_param( 'visible_'    . $i,   5, '');
      $is_auto    = bh_read_post_param( 'auto_'       . $i,   5, '');
      $day        = bh_read_post_param( 'day_'        . $i,   3, '');
      $schedule   = bh_read_post_param( 'schedule_'   . $i,  40, '');
      $anchor     = bh_read_post_param( 'anchor_'     . $i,  10, '');
      $start_time = bh_read_post_param( 'time_'       . $i,   4, '');
      $duration   = bh_read_post_param( 'duration_'   . $i,   5, '');
      $author     = bh_read_post_param( 'auth_'       . $i,  80, '');
      $playlist   = bh_read_post_param( 'list_'       . $i,  80, '');
      $auto_reset = bh_read_post_param( 'auto_reset_' . $i,   5, '');
      $day_mask   = bh_read_post_param( 'day_mask_'   . $i,   7, '');
      $clear_ptr  = bh_read_post_param( 'pointer_'    . $i,   5, '');

      if( ! isset( $is_visible) || ! $is_visible == 'on')
      {
        $is_visible = 'off';
      }

      if( ! isset( $is_auto) || ! $is_auto == 'on')
      {
        $is_auto = 'off';
      }

      if(
        ! isset( $day) ||
        ! (
          $day === "tue" ||
          $day === "wed" ||
          $day === "thu" ||
          $day === "fri" ||
          $day === "sat" ||
          $day === "sun"
        )
      )
      {
        $day = 'mon';
      }

      if(
        ! isset( $schedule) ||
        ! (
            $schedule === "daily"            ||
            $schedule === "week 1"           ||
            $schedule === "week 1 and 3"     ||
            $schedule === "week 2"           ||
            $schedule === "week 2 and 4"     ||
            $schedule === "week 3"           ||
            $schedule === "week 4"           ||
            $schedule === "week 5"           ||
            $schedule === "once"             ||
            $schedule === "every other week"
          )
      )
      {
        $schedule = 'weekly';
      }

      if( ! isset( $anchor) || ! bh_validate_anchor( $anchor, $day))
      {
        $anchor = '-';
      }

      if(
        ! isset( $start_time)         ||
        ! is_numeric( $start_time)    ||
        (((int) $start_time) <    0)  ||
        (((int) $start_time) > 2359)
      )
      {
        $start_time = $n_time;
      }

      if(
        ! isset( $duration)          ||
        ! is_numeric( $duration)     ||
        (((int) $duration) <     0)  ||
        (((int) $duration) > 527040)        // No playlists longer than one year, duration is in minutes
      )
      {
        $duration = $n_duration;
      }

      if( ! isset( $author) || ( strlen( $author) > 80))
      {
        $author = '';
      }

      if( ! isset( $playlist) || ( strlen( $playlist) > 80))
      {
        $playlist = '';
      }

      if( ! isset( $clear_ptr) || ! $clear_ptr == 'on')
      {
        $clear_ptr = 'off';
      }

      if( ! isset( $auto_reset) || ! $auto_reset == 'on')
      {
        $auto_reset = 'off';
      }

      if( ! isset( $day_mask) || ( strlen( $day_mask) != 7))
      {
        $day_mask = '1111111';
      }

      if( $clear_ptr == 'on' && $playlist != '')
      {
        $pointers_changed = true;
        $playlist_pointers[ $playlist] = 0;
      }

      $lines .= "\"$is_visible\",\"$is_auto\",\"$day\",\"$schedule\",\"$anchor\",\"$start_time\",\"$duration\",\"$author\",\"$playlist\",\"$auto_reset\",\"$day_mask\"\n";
    }

    // Save data to file

    // print( "\n$lines"); bh_local_debug( "save disabled for testing");

    if( file_put_contents( $bh_schedule_file, $lines, LOCK_EX) === false)
    {
      // bh_local_debug( "data may not have been saved ...");
    }
    else
    {
      bh_local_debug( "data was saved");
    }

    if( $pointers_changed)
    {
      // bh_local_debug( "some playlist pointers were cleared");
      bh_save_playlist_pointers( $bh_playlist_data_fn, $playlist_pointers);
    }
  }
  else
  {
    bh_local_debug( "data was loaded");
  }

  // Otherwise just don't save and serve back the same old data


  // bh_hit( 300);

  // --------------------------------------------------------------------------------------
  //
  // Read stored data into $data array for later use in JavaScript
  //
  // Stored data is a CSV file with the following data :
  //
  //   id, visible, auto, day, schedule, anchor, time, duration, author, playlist
  //
  // Note that "select" is not persistent.

  $text              = '';
  $bh_schedule       = bh_load_schedule( $bh_schedule_file);
  $playlist_pointers = bh_load_playlist_pointers( $bh_playlist_data_fn);

  // bh_hit( 400);

?><!DOCTYPE HTML>
<html>
  <head>
    <title>Playlist schedule editor</title>
    <link rel = "stylesheet"    href = "<?php if( $on_host) print( '../'); ?>styles/section.css">
    <link rel = "shortcut icon" href = "<?php if( $on_host) print( '../'); ?>images/bluehell.ico">
    <script type = "text/javascript">
        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        Object.prototype.clone =
          function()
          {
            var newObj = ( this instanceof Array) ? [] : {};

            for( i in this)
            {
              if( i == 'clone')
              {
                continue;
              }

              if( this[ i] && typeof this[ i] == "object")
              {
                newObj[ i] = this[i].clone();
              }
              else
              {
                newObj[ i] = this[ i];
              }
            }

            return newObj;
          };

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

          <?php
            // ////////////////////////////////////////////////////////////////

            // Create a JavaScript array to represent the file data

            $rows = count( $bh_schedule);    // bh_local_debug( "rows = $rows");
            if( $rows > 2)
            {
              $default_list     = $bh_schedule[ 0];                 // bh_debug( $default_list, 'default list');
              $default_author   = $default_list[ 'default_author'];
              $default_playlist = $default_list[ 'default_file'  ];
              $filler_list      = $bh_schedule[ 1];                 // bh_debug( $filler_list, 'filler list');
              $filler_author    = $filler_list[ 'filler_author'];
              $filler_playlist  = $filler_list[ 'filler_file'  ];
              unset( $bh_schedule[ 0]);
              unset( $bh_schedule[ 1]);
              $rows -= 2;
              $cols  = count( $bh_schedule[ 2]);                    // bh_local_debug( "cols = $cols");
            }
            else
            {
              $cols = 0;
            }
            print( "\n        var defaultAuthor     = '$default_author';"  );
            print( "\n        var defaultPlaylist   = '$default_playlist';");
            print( "\n");
            print( "\n        var fillerAuthor      = '$filler_author';"   );
            print( "\n        var fillerPlaylist    = '$filler_playlist';" );
            print( "\n");
            print( "\n        var entryDataOriginal = new Array( $rows);"  );
            print( "\n");
            print( "\n        for( var i = 0; i < $rows; i++)");
            print( "\n        {");
            print( "\n          entryDataOriginal[ i] = new Array( $cols);");
            print( "\n        }");
            print( "\n");

            $i = 0;
            foreach( $bh_schedule as $sched)
            {
              $j = 0;
              foreach( $sched as $key => $value)
              {
                $val = trim( $value);
                $val = str_replace( '"'  , '\"', $val);
                print( "\n        entryDataOriginal[ $i][ '$key'] = \"$val\";");
                $j++;
              }
              print(   "\n        entryDataOriginal[ $i][ 'selected'] = \"no\";"); $j++; // selected not saved, clear it
              $list = $sched[ 'file'];
              $pointer = $playlist_pointers[ $list];
              print(   "\n        entryDataOriginal[ $i][ 'pointer'] = \"$pointer\";"); $j++;
              $i++;
            }
            print( "\n");

            // ////////////////////////////////////////////////////////////////
          ?>

        var entryData;

        var weekDays      = new Array( "mon", "tue", "wed", "thu", "fri", "sat", "sun");
        var scheduleTypes = new Array( "daily", "weekly", "week 1", "week 1 and 3", "week 2", "week 2 and 4", "week 3", "week 4", "week 5", "once", "every other week");
        var playlists     = new Array(<?php
          print( "\n");
          $index = count( $bh_playlists);
          foreach( $bh_playlists as $name => $list)
          {
            print( "          \"$name\"");
            if( --$index > 0) print( ",");
            print( "\n");
          }
        ?>
        );

        function resetData()
        {
          entryData = entryDataOriginal.clone();
        }

        resetData();

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function itemCount()
        {
          var result = entryData.length;

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function removeDataElement( anIndex)
        {
          entryData.splice( anIndex, 1);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addDataElement(
          isVisible,
          isAuto,
          aDay,
          aSchedule,
          anAnchor,
          aStartTime,
          aDuration,
          anAuthor,
          aPlaylist,
          isAutoReset,
          aDayMask,
          isSelected,
          aPointer
        )
        {
          var aRow = new Array();

          aRow[ 'visible'   ] = isVisible;
          aRow[ 'playable'  ] = isAuto;
          aRow[ 'day'       ] = aDay;
          aRow[ 'type'      ] = aSchedule;
          aRow[ 'anchor'    ] = anAnchor;
          aRow[ 'start'     ] = aStartTime;
          aRow[ 'duration'  ] = aDuration;
          aRow[ 'author'    ] = anAuthor;
          aRow[ 'file'      ] = aPlaylist;
          aRow[ 'auto_reset'] = isAutoReset;
          aRow[ 'day_mask'  ] = aDayMask;
          aRow[ 'selected'  ] = isSelected;
          aRow[ 'pointer'   ] = aPointer;

          entryData.unshift( aRow);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function findElementById( anIndex)
        {
          var result = document.getElementById( anIndex);

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function syncSelections()
        {
          var e;

          for( var i = 0; i < itemCount(); i++)
          {
            e = findElementById( 'select_' + i)

            if( e.checked)
            {
              entryData[ i][ 'selected'] = 'on';
            }
            else
            {
              entryData[ i][ 'selected'] = '';
            }
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function removeDocElement( anIndex)
        {
          var i = '' + anIndex;
          var e = findElementById( i);
          e.parentNode.removeChild( e);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function hideDocElement( anIndex)
        {
          var i = '' + anIndex;
          var e = findElementById( i);
          if( e != null)
          {
            e.style.visibility = 'hidden';
          }
        }

          // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function insertDocElementAfter( aRefNode, aNewNode)
        {
          aRefNode.parentNode.insertBefore( aNewNode, aRefNode.nextSibling);
        }

          // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function insertDocElementBefore( aRefNode, aNewNode)
        {
          aRefNode.parentNode.insertBefore( aNewNode, aRefNode);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addTable( aParent)
        {
          var newControl;
          var aBody;

          newControl = document.createElement( 'table');
          aBody      = document.createElement( 'tbody');
          newControl.appendChild( aBody);
          aParent.appendChild( newControl);

          return aBody;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addTr( aParent)
        {
          var newControl;

          newControl = document.createElement( 'tr');
          aParent.appendChild( newControl);

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addTd( aParent)
        {
          var newControl;

          newControl = document.createElement( 'td');
          aParent.appendChild( newControl);

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createPara( aValue)
        {
          var newControl;

          newControl = document.createElement( 'p');
          newControl.innerHTML = aValue;

          return newControl;
        }


        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addPara( aParent, aValue)
        {
          var newControl;

          newControl = createPara( aValue);
          aParent.appendChild( newControl);

          return aParent;
        }


        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addText( aParent, aValue)
        {
          aParent.innerHTML = aParent.innerHTML + aValue;

          return aParent;
        }


        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createCheckBox( anIndex)
        {
          var newControl;

          newControl = document.createElement( 'input');

          newControl.setAttribute( 'type'   , 'checkbox');
          newControl.setAttribute( 'name'   , anIndex   );
          newControl.setAttribute( 'id'     , anIndex   );

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addCheckBox( aParent, anIndex, aValue)
        {
          var newControl;

          newControl = createCheckBox( anIndex);

          aParent.appendChild( newControl);  // Grmpf .. IE8 wants the control to be parented before it can be chhecked ...

          if( aValue == "on")
          {
            newControl.setAttribute( 'checked', 'on');
          }


          return aParent;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createInput( anIndex, aValue, aSize, anOnChanged)
        {
          var newControl;

          newControl = document.createElement( 'input');

          newControl.setAttribute( 'type' , 'text' );
          newControl.setAttribute( 'name' , anIndex);
          newControl.setAttribute( 'id'   , anIndex);
          newControl.setAttribute( 'value', aValue );
          newControl.setAttribute( 'size' , aSize  );

          newControl.onchange = anOnChanged;

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addInput( aParent, anIndex, aValue, aSize, anOnChanged)
        {
          var newControl;

          newControl = createInput( anIndex, aValue, aSize, anOnChanged) ;
          aParent.appendChild( newControl);

          return aParent;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createHidden( anIndex, aValue, anOnChanged)
        {
          var newControl;

          newControl = document.createElement( 'input');

          newControl.setAttribute( 'type' , 'hidden');
          newControl.setAttribute( 'name' , anIndex );
          newControl.setAttribute( 'id'   , anIndex );
          newControl.setAttribute( 'value', aValue  );

          newControl.onchange = anOnChanged;

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addHidden( aParent, anIndex, aValue, anOnChanged)
        {
          var newControl;

          newControl = createHidden( anIndex, aValue, anOnChanged);
          aParent.appendChild( newControl);

          return aParent;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createDropDown( anIndex, aValue, aValues, anOnChanged)
        {
          var newControl;
          var anOption;

          newControl = document.createElement( 'select');
          newControl.setAttribute( 'name' , anIndex);
          newControl.setAttribute( 'id'   , anIndex);

          for( var i = 0; i < aValues.length; i++)
          {
            anOption = document.createElement( 'option');
            newControl.appendChild( anOption);
            anOption.setAttribute( 'value', aValues[ i]);
            if( aValues[ i] == aValue)
            {
              anOption.setAttribute( 'selected', 'selected');
            }
            anOption.text = aValues[ i];

          }

          newControl.onchange = anOnChanged;

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addDropDown( aParent, anIndex, aValue, aValues, anOnChanged)
        {
          var newControl;

          newControl = createDropDown( anIndex, aValue, aValues, anOnChanged);
          aParent.appendChild( newControl);

          return aParent;
        }


        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createDaySelector( anIndex, aDayMask, anOnChanged)
        {
          return createInput( anIndex, aDayMask.toString( 2), 7, anOnChanged);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addDaySelector( aParent, anIndex, aDayMask, anOnChanged)
        {
          var newControl;

          var newControl = createDaySelector( anIndex, aDayMask, anOnChanged);
          aParent.appendChild( newControl);

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createDayPicker( anIndex, aDay, aSchedule)
        {
          if( aSchedule == 'daily')
          {
            return createHidden( anIndex, aDay, null);
          }
          else
          {
            return createDropDown( anIndex, aDay, weekDays, null);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createDayMaskPicker( anIndex, aDayMask, aSchedule)
        {
          if( aSchedule == 'daily')
          {
            return createDaySelector( anIndex, aDayMask, null);
          }
          else
          {
            return createHidden( anIndex, aDayMask.toString( 2), null);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addDayPicker( aParent, anIndex, aDay, aSchedule)
        {
          var newControl;

          var newControl = createDayPicker( anIndex, aDay, aSchedule);
          aParent.appendChild( newControl);

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addDayMaskPicker( aParent, anIndex, aDayMask, aSchedule)
        {
          var newControl;

          var newControl = createDayMaskPicker( anIndex, aDayMask, aSchedule);
          aParent.appendChild( newControl);

          return newControl;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function scheduleTypeChanged( anIndex)
        {
          // Schedule type changed, determine if anchor box should be visible
          // or not.

          var e = findElementById( anIndex);
          var i = anIndex.split( '_')[ 1];
          var s = e.options[ e.selectedIndex].value;
          var a = 'anchor_' + i;
          var v = findElementById( a).getAttribute( 'value');

          removeDocElement( a);
          if(( s == "once") || ( s == "every other week"))
          {
            insertDocElementAfter( e, createInput( a, v, 10, function(){ anchorChanged( a)}));
          }
          else
          {
            insertDocElementAfter( e, createHidden( a, v, function(){ anchorChanged( a)}));
          }

          var d = 'day_'      + i;
              v = findElementById( d).getAttribute( 'value');
          removeDocElement( d);
          insertDocElementBefore( e, createDayPicker( d, v, s));

          var m = 'day_mask_' + i;
              v = findElementById( m).getAttribute( 'value');
          removeDocElement( m);
          insertDocElementBefore( e, createDayMaskPicker( m, parseInt( v, 2), s));
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addSchedulePicker( aParent, anIndex, aValue)
        {
          return addDropDown( aParent, anIndex, aValue, scheduleTypes, function(){ scheduleTypeChanged( anIndex)});
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addPlaylistPicker( aParent, anIndex, aValue)
        {
          return addDropDown( aParent, anIndex, aValue, playlists, null);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function prettyWeekDay( aDay)
        {
          if     ( aDay == 'mon') { return 'Mon'}
          else if( aDay == 'tue') { return 'Tue'}
          else if( aDay == 'wed') { return 'Wed'}
          else if( aDay == 'thu') { return 'Thu'}
          else if( aDay == 'fri') { return 'Fri'}
          else if( aDay == 'sat') { return 'Sat'}
          else if( aDay == 'sun') { return 'Sun'}

          else if( aDay == 1    ) { return 'Mon'}
          else if( aDay == 2    ) { return 'Tue'}
          else if( aDay == 3    ) { return 'Wed'}
          else if( aDay == 4    ) { return 'Thu'}
          else if( aDay == 5    ) { return 'Fri'}
          else if( aDay == 6    ) { return 'Sat'}
          else if( aDay == 0    ) { return 'Sun'}

          else                    { return 'Inv'}
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function numericWeekDay( aDay)
        {
          if     ( aDay == 'mon') { return 1}
          else if( aDay == 'tue') { return 2}
          else if( aDay == 'wed') { return 3}
          else if( aDay == 'thu') { return 4}
          else if( aDay == 'fri') { return 5}
          else if( aDay == 'sat') { return 6}
          else if( aDay == 'sun') { return 0}
          else                    { return 0}
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function numericWeekDayMask( aDay)
        {
          return 1 << numericWeekDay( aDay);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function anchorChanged( anIndex)
        {
          // note : this is only called when the focus goes away from the changed element
          //        so we can safely check for consistencey here and alter the selected
          //        DOW if needed.


          elt    = findElementById( anIndex);
          i      = anIndex.split( '_')[ 1];
          fdow   = 'day_' + i;
          sel    = findElementById( fdow);
          dowSet = numericWeekDay( sel.options[ sel.selectedIndex].value);

          date   = elt.value;
          year   = date.substr( 0, 4);
          month  = date.substr( 5, 2) - 1
          day    = date.substr( 8, 2);

          if(
            ( year  >  2010) &&
            ( year  <  2038) &&
            ( month >=    0) &&
            ( month <    12) &&
            ( day   >     0) &&
            ( day   <    32)
          )
          {
            var time = new Date( year, month, day, 0, 0, 0);
            dowCalc = time.getDay();

            if( dowSet != dowCalc)
            {
              sel.selectedIndex = ( dowCalc + 6) % 7;
              alert( 'Changed selected day to match the week day for the filled in date');
            }
          }
          else
          {
            alert( date + ' is not a valid date');
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addAnchor( aParent, anIndex, aValue, aScheduleType)
        {
          if(
            ( aScheduleType == "every other week") ||
            ( aScheduleType == "once"            )
          )
          {
            // an anchor date is needed
            addInput( aParent, anIndex, aValue, 10, function(){ anchorChanged( anIndex)});
          }
          else
          {
            // no anchor is needed
            addHidden( aParent, anIndex, '-', function(){ anchorChanged( anIndex)});
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addPointerClear( aParent, anIndex, aValue)
        {
          addCheckBox( aParent, anIndex, false);
          addText( aParent, 'pointer = ' + aValue);
        }


        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addDocElement(
          anIndex,
          isVisible,
          isAuto,
          aDay,
          aSchedule,
          anAnchor,
          aStartTime,
          aDuration,
          anAuthor,
          aPlaylist,
          isAutoReset,
          aDayMask,
          aPointer,
          isSelected
        )
        {
          var entries  = findElementById( 'entries');
          var newRow   = document.createElement( 'tr');
          var aSubTable;
          var aSubRow;
          var aTd;

          newRow.setAttribute( 'id', anIndex);

          addCheckBox( addTd( newRow), 'visible_' + anIndex, isVisible);
          addCheckBox( addTd( newRow), 'auto_'    + anIndex, isAuto   );

          aSubTable = addTable( addTd( newRow));
            aSubRow = addTr( aSubTable);
              addInput( addTd( aSubRow), 'auth_'    + anIndex, anAuthor , 80, null);

            aSubRow = addTr( aSubTable);
              aTd = addTd( aSubRow);
              addDayPicker     ( aTd, 'day_'        + anIndex, aDay       , aSchedule      );
              addDayMaskPicker ( aTd, 'day_mask_'   + anIndex, aDayMask   , aSchedule      );
              addSchedulePicker( aTd, 'schedule_'   + anIndex, aSchedule                   );
              addAnchor        ( aTd, 'anchor_'     + anIndex, anAnchor   , aSchedule      );
              addInput         ( aTd, 'time_'       + anIndex, aStartTime , 4        , null);
              addInput         ( aTd, 'duration_'   + anIndex, aDuration  , 5        , null);
              addPlaylistPicker( aTd, 'list_'       + anIndex, aPlaylist                   );


            aSubRow = addTr( aSubTable);
              aTd = addTd( aSubRow);
              addCheckBox      ( aTd, 'auto_reset_' + anIndex, isAutoReset                 );
              addText          ( aTd, 'allways start on the first song (clears playlist pointer when this item gets scheduled)');

          addPointerClear( addTd( newRow), 'pointer_' + anIndex, aPointer  );
          addCheckBox    ( addTd( newRow), 'select_'  + anIndex, isSelected);

          entries.appendChild( newRow);
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addRowFromData( anIndex)
        {
          var isVisible   = entryData[ anIndex][ 'visible'    ];
          var isAuto      = entryData[ anIndex][ 'playable'   ];
          var aDay        = entryData[ anIndex][ 'day'        ];
          var aSchedule   = entryData[ anIndex][ 'type'       ];
          var anAnchor    = entryData[ anIndex][ 'anchor'     ];
          var aStartTime  = entryData[ anIndex][ 'start'      ];
          var aDuration   = entryData[ anIndex][ 'duration'   ];
          var anAuthor    = entryData[ anIndex][ 'author'     ];
          var aPlaylist   = entryData[ anIndex][ 'file'       ];
          var isAutoReset = entryData[ anIndex][ 'auto_reset' ];
          var aDayMask    = entryData[ anIndex][ 'day_mask'   ];
          var aPointer    = entryData[ anIndex][ 'pointer'    ];
          var isSelected  = entryData[ anIndex][ 'selected'   ];

          addDocElement(
            anIndex,
            isVisible,
            isAuto,
            aDay,
            aSchedule,
            anAnchor,
            aStartTime,
            aDuration,
            anAuthor,
            aPlaylist,
            isAutoReset,
            aDayMask,
            aPointer,
            isSelected
          );
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function addRowsFromData()
        {
          for( var r = 0; r < itemCount(); r++)
          {
            addRowFromData( r);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function removeAllDocRows()
        {
          for( var i = 0; i < itemCount(); i++)
          {
            removeDocElement( i);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function swapDataEntries( i, j)
        {
          var t         = entryData[ i];
          entryData[ i] = entryData[ j];
          entryData[ j] = t;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function isSelected( anIndex)
        {
          var result = entryData[ anIndex][ 'selected'] == 'on';

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function removeSelectedRow( anIndex)
        {
          if( isSelected( anIndex))
          {
            removeDataElement( anIndex);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function upSelectedRow( anIndex)
        {
          if( isSelected( anIndex) && anIndex > 0)
          {
            swapDataEntries( anIndex, anIndex - 1);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function downSelectedRow( anIndex)
        {
          if( isSelected( anIndex) && anIndex < itemCount() - 1)
          {
            swapDataEntries( anIndex, anIndex + 1);
          }
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function removeSelectedRows()
        {
          syncSelections();
          removeAllDocRows();
          var r = itemCount();
          while( r > 0)
          {
            removeSelectedRow( --r);
          }
          addRowsFromData();
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function upSelectedRows()
        {
          var r = 0;
          syncSelections();
          removeAllDocRows();
          while( r < itemCount())
          {
            upSelectedRow( r++);
          }
          addRowsFromData();
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function downSelectedRows()
        {
          var r = itemCount();
          syncSelections();
          removeAllDocRows();
          while( r > 0)
          {
            downSelectedRow( --r);
          }
          addRowsFromData();
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function showInfo( row)
        {
          author   = findElementById( 'auth_' + row).value;
          playlist = findElementById( 'list_' + row).value;
          result   = author + ', ' + playlist;

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function validateAnchor( anAnchor, aDow)
        {
          result = false;

          dowSet = numericWeekDay( aDow);
          year   = anAnchor.substr( 0, 4);
          month  = anAnchor.substr( 5, 2) - 1
          mday   = anAnchor.substr( 8, 2);

          if(
            ( year  >  2010) &&
            ( year  <  2038) &&
            ( month >=    0) &&
            ( month <    12) &&
            ( mday  >     0) &&
            ( mday  <    32)
          )
          {
            var time = new Date( year, month, mday, 0, 0, 0);
            dowCalc = time.getDay();

            result = dowSet == dowCalc;
          }
          else
          {
            result = false;
          }
          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function validateStartTime( aStartTime)
        {
          i = parseInt( aStartTime);
          result = ( i >= 0) && ( i < 2400);

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function validateDuration( aDuration)
        {
          i = parseInt( aDuration);
          result = ( i >= 0) && ( i < 527040); // no playlists longer than one year

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function validateForm()
        {
          result = true;
          form   = findElementById( 'main_form');
          msg    = '';

          for( var r = 0; r < itemCount(); r++)
          {
            aDow       = findElementById( 'day_'      + r).value;
            aSchedule  = findElementById( 'schedule_' + r).value;
            anAnchor   = findElementById( 'anchor_'   + r).value;
            aStartTime = findElementById( 'time_'     + r).value;
            aDuration  = findElementById( 'duration_' + r).value;

            if(( aSchedule == 'once') || ( aSchedule == 'every other week'))
            {
              if( ! validateAnchor( anAnchor, aDow))
              {
                result = false;
                msg    = 'error in anchor date "' + anAnchor + '" or selected day "' + aDow + '" does not match the anchor date';
                break;
              }
            }

            if( ! validateStartTime( aStartTime))
            {
              result = false;
              msg    = 'error in start time "' + aStartTime + '"';
              break;
            }

            if( ! validateDuration( aDuration))
            {
              result = false;
              msg    = 'error in duration "' + aDuration + '"';
              break;
            }
          }

          if( ! result)
          {
            alert( 'an error was detected on the playlist : "' + showInfo( r) + '" with message [' + msg + ']');
            result = false;
          }

          return result;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function saveForm()
        {
          findElementById( 'count').setAttribute( 'value', itemCount());
          if( validateForm())
          {
            findElementById( 'main_form').submit();
          }

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function reloadForm()
        {
          location.href = "<?php print( $this_url); ?>";

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function undoForm()
        {
          removeAllDocRows();
          resetData();
          addRowsFromData();

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function createNewEntry()
        {
          removeAllDocRows();
          addDataElement(
            'off',              // visible on schedule
            'off',              // Auto motd update
            'mon',              // day (mon .. sun)
            'weekly',           // schedule type  ( see scheduleTypes)
            '-',                // anchor         ( '-' or date, depending on schedule type)
            '0000',             // start time     ( GMT, as in 0100 i.e no colon)
            '60',               // duration       ( in minutes)
            '* new entry *',    // author
            '',                 // playlist
            'off',              // Auto_reset
            0x7f,               // Day mask
            'off',              // selected       (always off for a new entry)
            '0'                 // Pointer        (always zero for a new entry)
          );
          addRowsFromData();

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function deleteSelectedEntries()
        {
          removeSelectedRows();

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function moveSelectedUp()
        {
          upSelectedRows();

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        function moveSelectedDown()
        {
          downSelectedRows();

          return false;
        }

        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    </script>
  </head>

  <body>
    <div id = "noscript" align = "center">
      <table>
        <tr>
          <td>Please allow for JavaScript on this page ... it will not work without</td>
        </tr>
      </table>
    </div>
    <div id = "scripted" align = "center">
      <table id = "filler">
        <tbody>
          <tr>
            <td>&nbsp;</td>
          </tr>
          <tr>
            <td>&nbsp;</td>
          </tr>
          <tr>
            <td>&nbsp;</td>
          </tr>
        </tbody>
      </table>
      <form
        id       = "main_form"
        method   = "post"
        action   = "<?php print( $this_url); ?>"
      >
        <input id = "count" name = "count" type = "hidden" value = "0">
        <table id = "floater" style = "position:fixed; left: 27px; top: 0px;">
          <tbody>
            <tr>
              <td colspan = "6">
                Playlist Schedule editor<?php if( $debug_info !== "") { print( ", $debug_info");}?>
              </td>
              <td>
                <a href="playlist_feed_debug.php" target="playlist_debugger">debug</a> the playlist feed
              </td>
            </tr>
            <tr>
              <td width = "14%">
                <input type = "button" value = "add entry" onClick = "return createNewEntry();">
              </td>
              <td width = "14%">
                <input type = "button" value = "save" onClick = "return saveForm();">
              </td>
              <td width = "14%">
                <input type = "button" value = "reload" onClick = "return reloadForm();">
              </td>
              <td width = "14%">
                <input type = "button" value = "undo" onClick = "return undoForm();">
              </td>
              <td width = "14%">
                <input type = "button" value = "delete selected" onClick = "return deleteSelectedEntries();">
              </td>
              <td width = "14%">
                <input type = "button" value = "selected up" onClick = "return moveSelectedUp();">
              </td>
              <td width = "14%">
                <input type = "button" value = "selected down" onClick = "return moveSelectedDown();">
              </td>
            </tr>
          </tbody>
        </table>
        <table>
          <tbody id = "entries">

            <tr>
              <th colspan = "2">
                Default playlist
              </th>
              <td colspan = "3">
                <select
                  name  = "default_playlist"
                  id    = "default_playlist"
                ><?php
                  print( "\n");
                  foreach( $bh_playlists as $name => $list)
                  {
                    $selected = $name == $default_playlist;
                    if( $selected)
                      print( "                  <option value = \"$name\" selected = \"selected\">$name</option>\n");
                    else
                      print( "                  <option value = \"$name\">$name</option>\n");
                  }
                ?>
                </select>
                The default playlist will be played from when no other playlist is live.
              </td>
            </tr>

            <tr>
              <th colspan = "2">
                Author for default playlist
              </th>
              <td colspan = "3">
                <input
                  type  = "text"
                  name  = "default_author"
                  id    = "default_author"
                  value = "<?php print( $default_author); ?>"
                  size  = "80"
                >
              </td>
            </tr>

            <tr>
              <th colspan = "2">
                Filler playlist
              </th>
              <td colspan = "3">
                <select
                  name  = "filler_playlist"
                  id    = "filler_playlist"
                ><?php
                  print( "\n");
                  foreach( $bh_playlists as $name => $list)
                  {
                    $selected = $name == $filler_playlist;
                    if( $selected)
                      print( "                  <option value = \"$name\" selected = \"selected\">$name</option>\n");
                    else
                      print( "                  <option value = \"$name\">$name</option>\n");
                  }
                ?>
                </select>
                The filler playlist will be used to fill out gaps in case the current default song would run into a scheduled show.
              </td>
            </tr>

            <tr>
              <th colspan = "2">
                Author for filler playlist
              </th>
              <td colspan = "3">
                <input
                  type  = "text"
                  name  = "filler_author"
                  id    = "filler_author"
                  value = "<?php print( $filler_author); ?>"
                  size  = "80"
                >
              </td>
            </tr>

            <tr>
              <td colspan = "5">
                <hr>
              </td>
            </tr>

            <tr>
              <!-- <th>ID</th> -->
              <th width = "120">visible on schedule</th>
              <th width = "150">auto motd update</th>
              <th>
                author<br>
                day picker / schedule type [anchor] /start time / duration (minutes) / selected playlist<br>
              </th>
              <th width = "120">playlist pointer</th>
              <th width = "120">select</th>
            </tr>

            <tr>
              <td>
                This affects visibility in all tools, including visibility on the radio page.
                When not checked none of the tools will show this item.
              </td>
              <td>
                When this is not checked the playlist will be marked as [ cancelled thisweek ] in
                the schedule on the radio page.
              </td>
              <td>
                [anchor] is present only for schedule types needing it, and it will appear as
                needed.<br><br>
                When the anchor date is filled in and the focus moves from that field the
                weekday will be calculated from it and set appropriately (if the date was
                determined to be valid); click somewhere out of the anchor box for this.<br><br>

                When the schedule type is set to 'daily' the date picker will change into a day
                mask, use a 1 for a day that the schedule has to be on and a 0 for a day it has
                to be off. The week starts at mondy, so for a monday and sunday only show use 1000001.
              </td>
              <td>
                select to clear the playlist pointer, then hit Save (on the next schedule this item
                will then start with the first song again).
              </td>
              <td>select for delete / up / down operations</td>
            </tr>

          </tbody>
        </table>
      </form>
    </div>
  <script type = "text/javascript">
    removeDocElement( "noscript");
    hideDocElement( 'filler');
    addRowsFromData();
  </script>

</html>
